#include <gtest/gtest.h>
#include <gtest/gtest-death-test.h>
#include <QGSettings>
#include "../../registeredSession/plugins/wallpaper/wallpaperinterface.h"

class WallpaperInterfaceTest : public testing::Test
{
protected:
    static void SetUpTestSuite()
    {
        m_pWallpaperInterface = new WallpaperInterface();
    }

    static void TearDownTestSuite()
    {
    }

    static WallpaperInterface* m_pWallpaperInterface;
};

WallpaperInterface *WallpaperInterfaceTest::m_pWallpaperInterface = nullptr;

TEST_F(WallpaperInterfaceTest, getWallpaperMode)
{
    QGSettings gset("org.mate.background");
    QString oldValue = gset.get("picture-filename").toString();
    std::string curValue = m_pWallpaperInterface->getWallpaperMode().toStdString();
    if (oldValue.isEmpty()) {
        ASSERT_STREQ(curValue.c_str(), "color");
    } else {
        ASSERT_STREQ(curValue.c_str(), "picture");
    }
}

TEST_F(WallpaperInterfaceTest, setWallpaper)
{
    QGSettings gset("org.mate.background");
    QString oldValue = gset.get("picture-filename").toString();
    std::string wallpaperMode = m_pWallpaperInterface->getWallpaperMode().toStdString();
    if (!oldValue.isEmpty()) {
        m_pWallpaperInterface->setWallpaper("picture", "/usr/share/backgrounds/city.jpg");
        std::string newValue = gset.get("picture-filename").toString().toStdString();
        ASSERT_STREQ(newValue.c_str(), "/usr/share/backgrounds/city.jpg");
        m_pWallpaperInterface->setWallpaper("picture", oldValue.toStdString().c_str());
    } else {
        ASSERT_STREQ(wallpaperMode.c_str(), "color");
    }
}

TEST_F(WallpaperInterfaceTest, setDisplayType)
{
    std::string oldType = m_pWallpaperInterface->getDisplayType().toStdString();
    m_pWallpaperInterface->setDisplayType("centered");
    QGSettings gset("org.mate.background");
    std::string newType = gset.get("picture-options").toString().toStdString();
    ASSERT_STREQ(newType.c_str(), "centered");
    m_pWallpaperInterface->setDisplayType(oldType.c_str());
}

TEST_F(WallpaperInterfaceTest, resetDefault)
{
    QString oldType = m_pWallpaperInterface->getDisplayType();
    QGSettings gset("org.mate.background");
    std::string oldPicName = gset.get("picture-filename").toString().toStdString();

    m_pWallpaperInterface->resetDefault();

    std::string newType = m_pWallpaperInterface->getDisplayType().toStdString();
    ASSERT_STREQ(newType.c_str(), "scaled");

    std::string newPicName = gset.get("picture-filename").toString().toStdString();
    ASSERT_STREQ(newPicName.c_str(), "/usr/share/backgrounds/1-openkylin.jpg");

    m_pWallpaperInterface->setDisplayType(oldType);
    gset.set("picture-filename", oldPicName.c_str());
}

TEST_F(WallpaperInterfaceTest, getPreviewWallpapers)
{
    int size = m_pWallpaperInterface->property("previewWallpapers").toStringList().size();
    ASSERT_GE(size, 0);
}

TEST_F(WallpaperInterfaceTest, getSourceWallpapers)
{
    int size = m_pWallpaperInterface->property("sourceWallpapers").toStringList().size();
    ASSERT_GE(size, 0);
}
